﻿using System;
using System.Collections.Generic;
using System.Linq.Expressions;

namespace EFCodeFirstExtensions
{
  public interface IEFCodeFirstRepository : IDisposable
  {

    /// <summary>
    /// Select data from database using a where clause
    /// </summary>
    /// <typeparam name="TItem">Type of data to select</typeparam>
    /// <param name="whereClause">Where clause / function</param>
    /// <param name="orderBy">Order by clause</param>
    /// <param name="skip">Paging implementation = number of records to skip</param>
    /// <param name="top">Paging implementation = number of records to return</param>
    /// <param name="include">Navigation properties to include</param>
    /// <returns>Items selected</returns>
    IEnumerable<TItem> Select<TItem>(
      Expression<Func<TItem, bool>> whereClause = null,
      IOrderByClause<TItem>[] orderBy = null,
      int skip = 0,
      int top = 0,
      string[] include = null)
      where TItem : class, new();

    /// <summary>
    /// Update an item
    /// </summary>
    /// <typeparam name="TItem">Type of item to update</typeparam>
    /// <param name="item">Item to update</param>
    /// <param name="saveImmediately">If set to true, saved occurs right away</param>
    /// <returns>Updated item</returns>
    TItem Update<TItem>(TItem item, bool saveImmediately = true) where TItem : class, new();

    /// <summary>
    /// Delete an item
    /// </summary>
    /// <typeparam name="TItem">Type of item to delete</typeparam>
    /// <param name="saveImmediately">If set to true, saved occurs right away</param>
    /// <param name="item">Item to delete</param>
    void Delete<TItem>(TItem item, bool saveImmediately = true) where TItem : class, new();


    /// <summary>
    /// Insert new item into database
    /// </summary>
    /// <typeparam name="TItem">Type of item to insert</typeparam>
    /// <param name="item">Item to insert</param>
    /// <param name="saveImmediately">If set to true, saved occurs right away</param>
    /// <returns>Inserted item</returns>
    TItem Insert<TItem>(TItem item, bool saveImmediately = true) where TItem : class, new();

    /// <summary>
    /// Save all pending changes
    /// </summary>
    void Save();

  }
}
